﻿// NaverUriBuilder.cs
//

using System;
using System.Collections.Generic;
using System.Globalization;
using System.Linq;
using System.Linq.Expressions;
using System.Text;
using LinqProviders.LinqToNaver.Utility;
using LinqProviders.Utility;

namespace LinqProviders.LinqToNaver {

    internal sealed class NaverUriBuilder : ExpressionVisitor {

        private StringBuilder _urlBuilder;
        private Dictionary<string, string> _queryString;
        private List<string> _queryExtensions;

        private string query = string.Empty;

        private string _searchField;
        private string _parameterPrefix;

        public NaverUriBuilder(Type searchResultType, string appKey)
            : this(searchResultType, appKey, null) {
        }

        public NaverUriBuilder(Type searchResultType, string appKey, IDictionary<string, object> parameters) {
            _queryString = new Dictionary<string, string>();
            _queryString["key"] = appKey;
            _queryString["target"] = "";
            _queryString["query"] = "";
            _queryString["display"] = "10";
            _queryString["start"] = "1";
            //_queryString["sort"] = "sim"; //date, sim

            if (parameters != null) {
                foreach (KeyValuePair<string, object> parameter in parameters) {
                    _queryString[parameter.Key] = parameter.Value.ToString();
                }
            }

            if (searchResultType == typeof(RankSearchResult))
            {
                _queryString["target"] = "rank";
            }
            else if (searchResultType == typeof(ImageSearchResult)) {
                _queryString["target"] = "image";
            }
            else if (searchResultType == typeof(NewsSearchResult))
            {
                _queryString["target"] = "news";
            }
            else if (searchResultType == typeof(VideoSearchResult))
            {
                _queryString["target"] = "video";
            }
            else if (searchResultType == typeof(BlogSearchResult))
            {
                _queryString["target"] = "blog";
            }
           
            else if (searchResultType == typeof(RecmdSearchResult))
            {
                _queryString["target"] = "recmd";
            }
            else if (searchResultType == typeof(WebkrSearchResult))
            {
                _queryString["target"] = "webkr";
            }
            else if (searchResultType == typeof(KinSearchResult))
            {
                _queryString["target"] = "kin";
            }
            else if (searchResultType == typeof(DocSearchResult))
            {
                _queryString["target"] = "doc";
            }
            else if (searchResultType == typeof(ShopSearchResult))
            {
                _queryString["target"] = "shop";
            }
            else if (searchResultType == typeof(EncycSearchResult))
            {
                _queryString["target"] = "encyc";
            }
            else if (searchResultType == typeof(ErrataSearchResult))
            {
                _queryString["target"] = "errata";
            }
            else if (searchResultType == typeof(CafeSearchResult))
            {
                _queryString["target"] = "cafe";
            }





            _urlBuilder = new StringBuilder();
            _urlBuilder.Append("http://openapi.naver.com/search?");
        }

        public Uri BuildUri(Expression expression, out string query) {
            if (expression != null) {
                expression = PartialEvaluator.Eval(expression, delegate(Expression expr) {
                    if ((expr.NodeType == ExpressionType.Call) &&
                        ((MethodCallExpression)expr).Method.DeclaringType == typeof(NaverQueryable)) {
                        return false;
                    }

                    return (expr.NodeType != ExpressionType.Parameter);
                });
            }

            Visit(expression);

            if (_queryString.TryGetValue("query", out query) == false) {
                throw new InvalidOperationException("The query expression must contain a where expression.");
            }
            if (_queryExtensions != null) {
                foreach (string extension in _queryExtensions) {
                    query = query + "" + extension;
                }
                _queryString["query"] = query;
            }

            foreach (KeyValuePair<string, string> parameter in _queryString) {
                _urlBuilder.Append(parameter.Key);
                _urlBuilder.Append("=");
                _urlBuilder.Append(Uri.EscapeDataString(parameter.Value));
                _urlBuilder.Append("&");
            }

            

            string url = _urlBuilder.ToString();
            if (url.EndsWith("&"))
                url = url.Remove(url.Length - 1);

            return ConversionHelpers.ToUri(url, UriKind.Absolute);
        }



        protected override Expression VisitBinary(BinaryExpression b) {
            if ((b.NodeType == ExpressionType.Equal) ||
                (b.NodeType == ExpressionType.AndAlso)) {
                return base.VisitBinary(b);
            }

            throw new NotSupportedException("Only equality comparisons and logical ANDs are supported in binary expressions.");
        }

        protected override Expression VisitConditional(ConditionalExpression c) {
            throw new NotSupportedException("Conditional expressions are not supported in queries.");
        }

        protected override Expression VisitConstant(ConstantExpression c) {
            if (c.Value == null) {
                throw new ArgumentNullException("Cannot use null in queries.");
            }

            if (String.CompareOrdinal(_searchField, "Query") == 0) {
                _queryString["query"] = (string)c.Value;
                _searchField = null;
            }
            else if (String.CompareOrdinal(_searchField, "Start") == 0)
            {
                _queryString["start"] = c.Value.ToString();
                _searchField = null;
            }
            else if (String.CompareOrdinal(_searchField, "Display") == 0)
            {
                _queryString["display"] = c.Value.ToString();
                _searchField = null;
            }
            else if (String.CompareOrdinal(_searchField, "Sort") == 0)
            {
                _queryString["sort"] = (string)c.Value;
                _searchField = null;
            }
            else if (String.CompareOrdinal(_searchField, "Filter") == 0)
            {
                _queryString["filter"] = (string)c.Value;
                _searchField = null;
            }

            return c;
        }

        protected override Expression VisitMemberAccess(MemberExpression m) {
            if ((m.Expression != null) && (m.Expression.NodeType == ExpressionType.Parameter)) {
                string name = m.Member.Name;

                if (_queryString.ContainsKey(name)) {
                    throw new NotSupportedException(String.Format("The member '{0}' cannot appear more than once in the query expression.", name));
                }

                if (String.CompareOrdinal(name, "Query") == 0) {
                    _searchField = name;
                }
                else if (String.CompareOrdinal(name, "Start") == 0)
                {
                    _searchField = name;
                }
                else if (String.CompareOrdinal(name, "Display") == 0)
                {
                    _searchField = name;
                }
                else if (String.CompareOrdinal(name, "Sort") == 0)
                {
                    _searchField = name;
                }
                else if (String.CompareOrdinal(name, "Filter") == 0)
                {
                    _searchField = name;
                }
                else {
                    throw new InvalidOperationException("Only the Query member can be used in a query expression.");
                }
                return m;
            }

            throw new NotSupportedException(String.Format("The member '{0}' is not supported.", m.Member.Name));
        }

        protected override Expression VisitMethodCall(MethodCallExpression m)
        {
            if ((m.Method.DeclaringType == typeof(Queryable)) ||
                (m.Method.DeclaringType == typeof(Enumerable)))
            {
                if (String.CompareOrdinal(m.Method.Name, "Where") == 0)
                {
                    Visit(m.Arguments[0]);

                    LambdaExpression lambda = (LambdaExpression)StripQuotes(m.Arguments[1]);
                    Visit(lambda.Body);

                    return m;
                }
                if (String.CompareOrdinal(m.Method.Name, "Skip") == 0)
                {
                    Visit(m.Arguments[0]);

                    ConstantExpression countExpression = (ConstantExpression)StripQuotes(m.Arguments[1]);
                    _queryString[_parameterPrefix + "start"] = ((int)countExpression.Value).ToString(CultureInfo.InvariantCulture);

                    return m;
                }
                if (String.CompareOrdinal(m.Method.Name, "Take") == 0)
                {
                    Visit(m.Arguments[0]);

                    ConstantExpression countExpression = (ConstantExpression)StripQuotes(m.Arguments[1]);
                    _queryString[_parameterPrefix + "display"] = ((int)countExpression.Value).ToString(CultureInfo.InvariantCulture);

                    return m;
                }
            }
            else if (m.Method.DeclaringType == typeof(NaverQueryable))
            {
                if (String.CompareOrdinal(m.Method.Name, "Sort") == 0)
                {
                    Visit(m.Arguments[0]);

                    _queryString["Sort"] = "Strict";
                    return m;
                }
                else if (String.CompareOrdinal(m.Method.Name, "LocalResults") == 0)
                {
                    Visit(m.Arguments[0]);

                    ConstantExpression locationExpression = (ConstantExpression)StripQuotes(m.Arguments[1]);
                    if (_queryExtensions == null)
                    {
                        _queryExtensions = new List<string>();
                    }
                    _queryExtensions.Add("loc=" + (string)locationExpression.Value);

                    return m;
                }
                else if (String.CompareOrdinal(m.Method.Name, "ScopeResults") == 0)
                {
                    Visit(m.Arguments[0]);

                    ConstantExpression siteExpression = (ConstantExpression)StripQuotes(m.Arguments[1]);
                    if (_queryExtensions == null)
                    {
                        _queryExtensions = new List<string>();
                    }
                    _queryExtensions.Add("&domain=" + (string)siteExpression.Value);

                    return m;
                }
            }

            throw new NotSupportedException(String.Format("The method '{0}' is not supported.", m.Method.Name));
        }

        protected override NewExpression VisitNew(NewExpression nex) {
            throw new NotSupportedException("New expressions are not supported in queries.");
        }

        protected override Expression VisitUnary(UnaryExpression u) {
            throw new NotSupportedException("Unary expressions are not supported in queries.");
        }
    }
}
